/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.jira.mock;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

import org.codehaus.plexus.util.IOUtil;
import org.codehaus.swizzle.jira.Attachment;
import org.sonatype.inject.Nullable;
import org.sonatype.jira.AttachmentHandler;
import org.sonatype.jira.AttachmentResponse;
import org.sonatype.jira.internal.JIRARestAttachmentHandler;

@Named( "mock" )
@Singleton
public class MockAttachmentHandler
    implements AttachmentHandler
{

    private StubJira mock;

    @Override
    public AttachmentResponse attach( final String url, final String username, final String password,
                                      final String issueId, final String issueKey,
                                      final Collection<org.sonatype.jira.Attachment> items )
        throws IOException
    {
        Map<Attachment, byte[]> files = new HashMap<Attachment, byte[]>();
        for ( org.sonatype.jira.Attachment item : items )
        {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            IOUtil.copy( item.getInputStream(), out );
            Attachment att = new Attachment();
            att.setAuthor( username );
            att.setFileName( item.getName() );
            att.setCreated( new Date() );
            files.put( att, out.toByteArray() );
        }

        if ( this.mock == null )
        {
            // no mock set means this is client-side injected and no 'current' mock bound
            new JIRARestAttachmentHandler().attach( url, username, password, issueId, issueKey, items );
        }
        else
        {
            mock.addAttachments( issueKey, files );
        }

        return new AttachmentResponse( 200, "attachments set", null );
    }

    @Inject
    public void setMock( final @Nullable @Named( "current" ) StubJira mock )
    {
        this.mock = mock;
    }
    

    @Override
    public boolean isSupporting( final String version )
    {
        return version.endsWith( "MOCK" );
    }

}
