/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.jira.mock;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

import javax.inject.Named;
import javax.inject.Singleton;

import org.apache.xmlrpc.XmlRpcException;
import org.codehaus.swizzle.jira.Attachment;
import org.codehaus.swizzle.jira.Issue;
import org.codehaus.swizzle.jira.MapObject;
import org.codehaus.swizzle.jira.Project;
import org.sonatype.jira.JiraXmlRpcProxy;

import com.thoughtworks.xstream.XStream;

@SuppressWarnings( { "unchecked", "rawtypes" } )
@Named
@Singleton
public class StubJira
    implements JiraXmlRpcProxy
{

    public StubJira()
    {
        super();
        // TODO Auto-generated constructor stub
    }

    private static final XStream XSTREAM;

    private Database db = new Database();

    static
    {
        XSTREAM = new XStream();
        XSTREAM.omitField( MapObject.class, "formats" );
    }


    @Override
    public boolean addComment( final String token, final String issueKey, final String comment )
    {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public Hashtable createIssue( final String token, final Hashtable rIssueStruct )
    {
        checkToken( token );
        int idx = db.issues.size() + 1;

        Issue issue = new Issue( rIssueStruct );

        String project = issue.getProject().getKey();
        String key = project + "-" + idx;
        issue.setKey( key );
        issue.setId( idx );
        issue.setLink( "MOCK" );
        issue.setUpdated( new Date() );

        Hashtable<String, Object> createdHT = new Hashtable<String, Object>( issue.toMap() );
        db.issues.put( key, createdHT );
        return createdHT;
    }

    @Override
    public Vector getComments( final String token, final String issueKey )
    {
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException();
    }

    @Override
    public Vector getComponents( final String token, final String projectKey )
    {
        checkToken( token );
        return db.components.get( projectKey );
    }

    @Override
    public Vector getFavouriteFilters( final String token )
    {
        return new Vector();
    }

    @Override
    public Hashtable getIssue( final String token, final String issueKey )
    {
        checkToken( token );
        return db.issues.get( issueKey );
    }

    @Override
    public Vector getIssuesFromFilter( final String token, final String filterId )
    {
        return new Vector();
    }

    @Override
    public Vector getIssuesFromTextSearch( final String token, final String searchTerms )
    {
        Vector keys = new Vector();
        keys.add( ".*" );
        return getIssuesFromTextSearchWithProject( token, keys, searchTerms, Integer.MAX_VALUE );
    }

    @Override
    public Vector getIssuesFromTextSearchWithProject( final String token, final Vector projectKeys,
                                                      final String searchTerms, final int maxNumResults )
    {
        // TODO add capability to use predefined search results
        checkToken( token );

        String term = searchTerms;
        if ( searchTerms.startsWith( "\"" ) && searchTerms.endsWith( "\"" ) )
        {
            term = searchTerms.substring( 1, searchTerms.length() - 1 );
        }

        Vector results = new Vector();
        for ( Hashtable ht : db.issues.values() )
        {
            Issue issue = new Issue( ht );
            for ( Object key : projectKeys )
            {
                if ( issue.getProject().getKey().matches( key.toString() ) )
                {
                    if ( term.equals( issue.getSummary() ) )
                    {
                        results.add( issue.toMap() );
                    }
                    else if ( issue.getDescription() != null && issue.getDescription().contains( term ) )
                    {
                        results.add( issue.toMap() );
                    }
                }
            }
        }
        return results;
    }

    @Override
    public Vector getIssueTypes( final String token )
    {
        checkToken( token );
        return db.issueTypes;
    }

    @Override
    public Vector getIssueTypesForProject( final String token, final String projectId )
    {
        return getIssueTypes( token );
    }

    @Override
    public Vector getPriorities( final String token )
    {
        checkToken( token );
        return db.priorities;
    }

    @Override
    public Vector getProjectsNoSchemes( final String token )
    {
        checkToken( token );

        Vector ret = new Vector();

        Collection<Project> projects = db.projects.values();
        for ( Project project : projects )
        {
            Map map = project.toMap();
            ret.add( map );
        }

        return ret;
    }

    @Override
    public Vector getResolutions( final String token )
    {
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException();
    }

    @Override
    public Vector getSavedFilters( final String token )
    {
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException();
    }

    @Override
    public Hashtable getServerInfo( final String token )
    {

        Hashtable info = new Hashtable();
        info.put( "version", "4.0.0-MOCK" );
        info.put( "baseUrl", "http://mock.invalid" );
        return info;
    }

    @Override
    public Vector getStatuses( final String token )
    {
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException();
    }

    @Override
    public Vector getSubTaskIssueTypes( final String token )
    {
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException();
    }

    @Override
    public Vector getSubTaskIssueTypesForProject( final String token, final String projectId )
    {
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException();
    }

    @Override
    public Hashtable getUser( final String token, final String username )
    {
        checkToken( token );
        Hashtable<String, Object> info = db.userInfo.get( username );
        if ( info == null )
        {
            throw new RuntimeException( "no such user found" );
        }
        return info;
    }

    @Override
    public Vector getVersions( final String token, final String projectKey )
    {
        checkToken( token );
        return db.versions.get( projectKey );
    }

    @Override
    public String login( final String username, final String password )
    {
        if ( username.equals( "invalid" ) )
        {
            throw new IllegalArgumentException( "login failure" );
        }

        // if ( password.equals( db.users.get( username ) ) )
        // {
        String token = username + Long.toHexString( System.currentTimeMillis() );
        db.tokens.add( token );
        Hashtable<String, Object> info = new Hashtable<String, Object>();
        info.put( "name", username );
        db.userInfo.put( username, info );

        return token;
        // }
        // else
        // {
        // throw new RuntimeXmlRpcException( new XmlRpcException( 403, "Wrong user/password" ) );
        // }
    }

    @Override
    public Boolean logout( final String token )
    {
        return db.tokens.remove( token );
    }

    @Override
    public Hashtable updateIssue( final String token, final String issueKey, final Hashtable fieldValues )
    {
        checkToken( token );
        checkIssue( issueKey );
        Hashtable<String, Object> hash = db.issues.get( issueKey );
        hash.putAll( fieldValues );
        Issue issue = new Issue( hash );
        issue.setLink( "MOCK" );
        issue.setUpdated( new Date() );
        hash = new Hashtable( issue.toMap() );
        db.issues.put( issueKey, hash );
        return hash;
    }

    public String getCompatibleVersion()
    {
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException();
    }

    @Override
    public void setProxiedUrl( final String url )
    {
        // TODO Auto-generated method stub

    }

    private void checkIssue( final String issueKey )
    {
        if ( !db.issues.containsKey( issueKey ) )
        {
            exception( 404, "No such issue" );
        }
    }

    private void checkToken( final String token )
    {
        if ( !db.tokens.contains( token ) )
        {
            exception( 403, "Invalid token" );
        }
    }

    private void exception( final int pCode, final String pMessage )
    {
        throw new RuntimeXmlRpcException( new XmlRpcException( pCode, pMessage ) );
    }

    /* Additional methods to set up test database */

    public void createUser( final String username, final String password )
    {
        this.db.users.put( username, password );
    }

    public String getDatabase()
    {
        return XSTREAM.toXML( this.db );
    }

    public void setDatabase( final String xml )
    {
        this.db = (Database) XSTREAM.fromXML( xml );
    }

    public void setComponents( final String projectKey, final Vector components )
    {
        db.components.put( projectKey, components );
    }

    public void setVersions( final String projectKey, final Vector versions )
    {
        db.versions.put( projectKey, versions );
    }

    public void addAttachments( final String issueKey, final Map<Attachment, byte[]> files )
    {

        Map<Attachment, byte[]> attachments;
        if ( ( attachments = db.attachments.get( issueKey ) ) == null )
        {
            attachments = new HashMap<Attachment, byte[]>();
            db.attachments.put( issueKey, attachments );
        }

        attachments.putAll( files );
    }

    public void addProject( final String projectKey, final Hashtable values )
    {
        db.projects.put( projectKey, new Project( values ) );
        setVersions( projectKey, new Vector() );
        setComponents( projectKey, new Vector() );
    }

    public void setIssueTypes( final Vector issueTypes )
    {
        db.issueTypes = issueTypes;
    }

    public Map<Attachment, byte[]> getAttachments( final String issueKey )
    {
        return db.attachments.get( issueKey );
    }

    public void setPriorities( final Vector<Hashtable> priorities )
    {
        db.priorities = priorities;
    }

    public Hashtable<String, Object> getIssue( final String issueKey )
    {
        return db.issues.get( issueKey );
    }

    @Override
    public String getProxiedUrl()
    {
        return "http://mock.invalid";
    }
}
